/*
 * Creator: Rosalva Gallardo-Valencia
 * Course: Inf111, Winter 2009
 * 
 * Created on Feb 28, 2009
 * 
 * Copyright, 2009 University of California. 
 * 
 */
package edu.uci.ics.inf111.dvdvendor.integrationtests;

import java.text.ParseException;
import java.util.Enumeration;

import edu.uci.ics.inf111.dvdvendor.app.BarCode;
import edu.uci.ics.inf111.dvdvendor.app.CheckOutCart;
import edu.uci.ics.inf111.dvdvendor.app.DVDVendor;
import edu.uci.ics.inf111.dvdvendor.app.Product;
import edu.uci.ics.inf111.dvdvendor.app.ProductDB;
import edu.uci.ics.inf111.dvdvendor.app.TransactionDB;
import edu.uci.ics.inf111.dvdvendor.devices.DVDDispenser;
import edu.uci.ics.inf111.dvdvendor.devices.PaymentCollector;
import edu.uci.ics.inf111.dvdvendor.exceptions.AddWhileDispensingException;
import edu.uci.ics.inf111.dvdvendor.exceptions.AddWhilePayingException;
import edu.uci.ics.inf111.dvdvendor.exceptions.IncorrectStateException;
import edu.uci.ics.inf111.dvdvendor.exceptions.InvalidBarCodeException;
import edu.uci.ics.inf111.dvdvendor.exceptions.InvalidProductException;
import edu.uci.ics.inf111.dvdvendor.exceptions.PayWhileDispensingException;
import edu.uci.ics.inf111.dvdvendor.exceptions.PayWithNoItemsException;
import edu.uci.ics.inf111.dvdvendor.exceptions.PaymentCollectorException;
import edu.uci.ics.inf111.dvdvendor.exceptions.TransactionDBException;
import junit.framework.TestCase;

public class DVDVendorTest extends TestCase {
	
	DVDVendor firstDVDVendor;
	DVDDispenser firstDD;
	ProductDB firstPDB;
	TransactionDB firstTDB;
	PaymentCollector firstPC;	
	BarCode firstBarCode, secondBarCode;
	String firstCardNumber;

	
	public DVDVendorTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		
		//create a DVDVendor
		firstDD = new DVDDispenser();
		firstPC = new PaymentCollector();
		firstPDB = ProductDB.getInstance();
		firstTDB = new TransactionDB();

		try {
			firstTDB.initializeTestDB();
		} catch (Exception e) {
			throw (e);
		}

		firstDVDVendor = new DVDVendor(firstDD, firstPC, firstPDB, firstTDB);

		// create BarCodes to include in our tests
		try {
			firstBarCode = new BarCode("024543213710");
			secondBarCode = new BarCode("071009003507");
		} catch (InvalidBarCodeException e) {
			fail("Invalid BarCode");
		}
		firstCardNumber = "1234567890123456";
		
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		
		firstDVDVendor = null;
		firstDD = null;
		firstPC = null;
		firstPDB = null;
		firstTDB = null;
		firstCardNumber = null;

	}

	public void testDVDVendor() {
		DVDVendor noArgDVDVendor;
		
		try {
			noArgDVDVendor = new DVDVendor();
			assertNotNull(noArgDVDVendor.getDVDDispenser());
			assertNotNull(noArgDVDVendor.getPaymentCollector());
			assertNotNull(noArgDVDVendor.getProductDB());
			assertNotNull(noArgDVDVendor.getTransactionDB());
			
		} catch (Exception e) {
			fail("No-arg constructor for DVDVendor failed");
		}
	}
	/*
	 * Don't need this, because it's been called in the setUp
	 *
		public void testDVDVendorDVDDispenserPaymentCollectorProductDBTransactionDB() {
			fail("Not yet implemented");
		}
	 */

	
	public void testInitiateRental() {
		try {
			
			//Check if the state is READY
			assertEquals(firstDVDVendor.getTransactionState(), firstDVDVendor.getTransactionState().READY);
		
			firstDVDVendor.initiateRental();
			
			//Check if the state is ADDING
			assertEquals(firstDVDVendor.getTransactionState(), firstDVDVendor.getTransactionState().ADDING);
				
			
		} catch (IncorrectStateException ise) {
			fail("Invalid action for current state of DVDVendor.");
		}	
	}

	//	This test case has been left as an exercise for the students.
	public void testAddItem() {
		fail("Not yet implemented");
	}

	public void testPayForRental() {
		
		CheckOutCart coc;
		Product product;
		double unitPrice;
		
		// now add something to the cart and do it again
		try {
			firstDVDVendor.initiateRental();
			firstDVDVendor.addItem(firstBarCode);

			// first check that coc contains some products
			coc = firstDVDVendor.payForRental(firstCardNumber);
			assertTrue(coc.listItems().hasMoreElements());

			// now check that the one Product in coc is the one that we put
			// in by checking the code and price
			product = coc.listItems().nextElement();
			assertEquals(firstBarCode.getBarCode(), product.getBarCode().getBarCode());

			unitPrice = firstPDB.lookUpItem(firstBarCode).getPrice();
			
			assertEquals(unitPrice, product.getPrice(), 0.0);


		} catch (PayWhileDispensingException awbe) {
			fail("Payment processed when the system was dispensing.");
		} catch (PayWithNoItemsException awpe) {
			fail("Payment processed when no items were added to the transaction.");
		} catch (PaymentCollectorException ippcee) {
			fail("Exception while collecting payment.");
		} catch (InvalidProductException ipe) {
			fail("Item not recognized.");
		} catch (IncorrectStateException ise) {
			fail("Invalid action for current state of DVDVendor.");
		} catch (TransactionDBException tdbe) {
			fail("Exception in TransactionDB.");
		}  catch (ParseException pe) {
			fail("Exception when calculating time difference between transactions.");
		}
	}

	//	This test case has been left as an exercise for the students.
	public void testDispenseItems() {
		fail("Not yet implemented");
	}

	public void testListItemsInCart() {
		Enumeration<Product> listOfProducts;
		Product product;

		// there's nothing in the cart, so this should be null
		listOfProducts = firstDVDVendor.listItemsInCart();
		assertFalse(listOfProducts.hasMoreElements());

		listOfProducts = null;

		// now add something to the cart and do it again
		try {
			firstDVDVendor.initiateRental();
			product = firstDVDVendor.addItem(firstBarCode);

			listOfProducts = firstDVDVendor.listItemsInCart();
			assertEquals(product, listOfProducts.nextElement());
			assertFalse(listOfProducts.hasMoreElements());

		} catch (AddWhileDispensingException awde) {
			fail("Item added when the system was dispensing.");
		} catch (AddWhilePayingException awpe) {
			fail("Item added when the system was processing payment");
		} catch (InvalidProductException ipe) {
			fail("Item not recognized.");
		} catch (IncorrectStateException ise) {
			fail("Invalid action for current state of DVDVendor.");
		} catch (TransactionDBException tdbe) {
			fail("Exception in TransactionDB.");
		}  catch (ParseException pe) {
			fail("Exception when calculating time difference between transactions.");
		}
		
	}

	public void testGetTotalCost() {

		double total, unitPrice;	
		assertEquals(0.0, firstDVDVendor.getTotalCost(), 0.0);
		
		// now add something to the cart and do it again
		try {
			firstDVDVendor.initiateRental();
			firstDVDVendor.addItem(firstBarCode);

			total = firstDVDVendor.getTotalCost();
			unitPrice = firstPDB.lookUpItem(firstBarCode).getPrice();
			
			assertEquals(unitPrice, total, 0.0);

		} catch (AddWhileDispensingException awde) {
			fail("Item added when the system was dispensing.");
		} catch (AddWhilePayingException awpe) {
			fail("Item added when the system was processing payment");
		} catch (InvalidProductException ipe) {
			fail("Item not recognized.");
		} catch (IncorrectStateException ise) {
			fail("Invalid action for current state of DVDVendor.");
		} catch (TransactionDBException tdbe) {
			fail("Exception in TransactionDB.");
		}  catch (ParseException pe) {
			fail("Exception when calculating time difference between transactions.");
		}
	}

	//	This test case has been left as an exercise for the students.
	public void testNotifyDVDDispenserEvent() {
		fail("Not yet implemented");
	}

	public void testGetDVDDispenser() {
		assertSame(firstDD, firstDVDVendor.getDVDDispenser());
	}

	public void testGetProductDB() {
		assertSame(firstPDB, firstDVDVendor.getProductDB());
	}

	public void testGetTransactionDB() {
		assertSame(firstTDB, firstDVDVendor.getTransactionDB());
	}

	public void testGetPaymentCollector() {
		assertSame(firstPC, firstDVDVendor.getPaymentCollector());
	}

}
